#include "SpriteComponent.h"
#define  STB_IMAGE_IMPLEMENTATION
#include <stb_image.h>
#include <GL/glew.h>
#include <iostream>

SpriteComponent::SpriteComponent(Object* owner) : Component(owner) {}

unsigned int SpriteComponent::LoadTexture(const std::string& path) {
    int w, h, channels;
    stbi_set_flip_vertically_on_load(1);
    unsigned char* data = stbi_load(path.c_str(), &w, &h, &channels, 4);
    if (!data) {
        std::cerr << "Failed to load image: " << path << std::endl;
        return 0;
    }

    size = glm::vec2(w,h);

    unsigned int id;
    glGenTextures(1, &id);
    glBindTexture(GL_TEXTURE_2D, id);

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
    glGenerateMipmap(GL_TEXTURE_2D);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

    stbi_image_free(data);
    return id;
}

void SpriteComponent::SetTexture(const std::string& path) {
    texturePath = path;
    textureID = LoadTexture(path);
}

void SpriteComponent::SetNormalMap(const std::string& path) {
    normalMapPath = path;
    normalMapID = LoadTexture(path);
}

unsigned int SpriteComponent::GetTextureID() const {
    return textureID;
}

unsigned int SpriteComponent::GetNormalMapID() const {
    return normalMapID;
}

std::string SpriteComponent::GetTexturePath() const {
    return texturePath;
}

std::string SpriteComponent::GetNormalMapPath() const {
    return normalMapPath;
}

void SpriteComponent::Save(YAML::Emitter& out) const {
    out << YAML::BeginMap;
    out << YAML::Key << "type" << YAML::Value << "SpriteComponent";
    out << YAML::Key << "texture" << YAML::Value << texturePath;
    out << YAML::Key << "normalMap" << YAML::Value << normalMapPath;
    out << YAML::EndMap;
}

void SpriteComponent::Load(const YAML::Node& node) {
    if (node["texture"]) SetTexture(node["texture"].as<std::string>());
    if (node["normalMap"]) SetNormalMap(node["normalMap"].as<std::string>());
}
