#pragma once
#include <string>
#include <vector>
#include <memory>
#include <glm/glm.hpp>
#include <yaml-cpp/yaml.h>

#include "../Components/Component.h"

class Object {
public:
    Object(const std::string& name);
    ~Object();

    void SetParent(Object* newParent);
    void AddChild(std::shared_ptr<Object> child);
    void RemoveChild(Object* child);

    glm::vec2 GetWorldPosition() const;
    glm::vec2 GetLocalPosition() const;
    void SetLocalPosition(glm::vec2 pos);

    const std::string& GetName() const;
    void SetName(const std::string& name);

    std::vector<std::shared_ptr<Object>>& GetChildren();
    Object* GetParent() const;

    template<typename T>
    std::shared_ptr<T> GetComponent() const;

    template<typename T>
    std::shared_ptr<T> AddComponent();

    template<typename T>
    void RemoveComponent();

    void Save(YAML::Emitter& out) const;
    void Load(const YAML::Node& node);

    int id;
    bool selected = false;

    int layer;

private:
    std::string name;
    glm::vec2 localPosition;
    Object* parent = nullptr;
    std::vector<std::shared_ptr<Object>> children;
    std::vector<std::shared_ptr<Component>> components;
};
