#define WIN32_LEAN_AND_MEAN
#define NOMINMAX
#include <windows.h>
#include <commdlg.h>
#include <unordered_map>
#include <string>
#include <filesystem>
#include "FileDialog.h"

static std::unordered_map<FileDialogType, std::string> filters = {
    { FileDialogType::Images, "Image Files\0*.png;*.jpg;*.jpeg;*.bmp\0" },
    { FileDialogType::Scenes, "CreateScene Files\0*.cene\0" },
    { FileDialogType::Audio, "Audio Files\0*.mp3;*.wav;*.ogg\0" },
    { FileDialogType::All, "All Files\0*.*\0" }
};

std::string OpenFileDialog(FileDialogType type) {
    char file[260] = { 0 };
    OPENFILENAMEA ofn = {};
    ofn.lStructSize = sizeof(ofn);
    ofn.lpstrFilter = filters.count(type) ? filters[type].c_str() : filters[FileDialogType::All].c_str();
    ofn.lpstrFile = file;
    ofn.nMaxFile = sizeof(file);
    ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR;

    auto current = std::filesystem::current_path();
    bool result = GetOpenFileNameA(&ofn);
    std::filesystem::current_path(current);
    return result ? std::string(file) : "";
}

std::string SaveFileDialog(FileDialogType type) {
    char file[260] = { 0 };
    OPENFILENAMEA ofn = {};
    ofn.lStructSize = sizeof(ofn);
    ofn.lpstrFilter = filters.count(type) ? filters[type].c_str() : filters[FileDialogType::All].c_str();
    ofn.lpstrFile = file;
    ofn.nMaxFile = sizeof(file);
    ofn.Flags = OFN_OVERWRITEPROMPT | OFN_NOCHANGEDIR;

    auto current = std::filesystem::current_path();
    bool result = GetSaveFileNameA(&ofn);
    std::filesystem::current_path(current);
    return result ? std::string(file) : "";
}
