#pragma once

#include <string>
#include <unordered_map>
#include <glm/glm.hpp>
#include <gl/glew.h>

class Shader {
public:
    Shader();
    ~Shader();

    bool LoadFromFile(const std::string& vertexPath, const std::string& fragmentPath);
    bool LoadFromSource(const std::string& vertexSrc, const std::string& fragmentSrc);

    void Use() const;
    GLuint GetID() const;

    // Setters
    void SetBool(const std::string& name, bool value);
    void SetInt(const std::string& name, int value);
    void SetFloat(const std::string& name, float value);
    void SetVec2(const std::string& name, const glm::vec2& value);
    void SetVec3(const std::string& name, const glm::vec3& value);
    void SetVec4(const std::string& name, const glm::vec4& value);
    void SetMat4(const std::string& name, const glm::mat4& mat);

private:
    GLuint id = 0;
    std::unordered_map<std::string, GLint> uniformCache;

    std::string ReadFile(const std::string& path);
    GLuint Compile(GLenum type, const std::string& source);
    GLint GetUniformLocation(const std::string& name);
    void Clear();
};
